#!/bin/bash

NODE=$(date +%s$RANDOM)
: ${ART_LOGIN:=_internal}
: ${ART_PASSWORD:=b6a50c8a15ece8753e37cbe5700bf84f}

function setPorts {
	# Artifactory's membership port, the PORT0 et PORT1 are specific to MESOS
	: ${MEMBERSHIP_PORT:=10042}
	: ${INSTANCE_PORT:=8081}
	if [ ! -z "$PORT0" ]; then
	    INSTANCE_PORT=$PORT0
	fi
	if [ ! -z "$PORT1" ]; then
	    MEMBERSHIP_PORT=$PORT1
	fi

	# Change the port
	sed -i -e "s,8081,$INSTANCE_PORT,g" /opt/jfrog/artifactory/tomcat/conf/server.xml
}

function setHaNodeProperties {
	local ha_home=/var/opt/jfrog/cluster
	cat >/etc/opt/jfrog/artifactory/ha-node.properties <<EOF
node.id="$NODE"
cluster.home=$ha_home
primary=false
context.url=http://$(hostname -I|awk '{print $1}'):$INSTANCE_PORT/artifactory
membership.port=$MEMBERSHIP_PORT
EOF
}

function setPrimaryBaseUrl {
	# See how we can deduce where the PRIMARY can be reached
	# One option can be : do a plain SQL request... to the artifactory_servers...
	# Or better try to ask the MESOS DNS for that, so :
	# If the ART_PRIMARY_BASE_URL is specified we'll use that, if not we will try to resolve it
	# from mesos DNS (if we want to make it generic we need to pass a variable to specify a DNS)
	if [ -z "$ART_PRIMARY_BASE_URL" ]; then
		ART_PRIMARY_PORT=$(curl "http://master.mesos:8123/v1/services/_instance._artifactory-primary._tcp.marathon.mesos" | jq -r '.[0] | .port')
	    ART_PRIMARY_BASE_URL=http://artifactory-primary.marathon.mesos:$ART_PRIMARY_PORT/artifactory
	fi
}

function waitForPrimaryNode {
	log "WAITING FOR PRIMARY NODE : $ART_PRIMARY_BASE_URL"
	local arguments=" --output /dev/null --silent --fail $ART_PRIMARY_BASE_URL/api/system/ping"
	local curl=$(curlAuth)
	until $(eval $curl $arguments)
	do
		echo "."
		sleep 2
	done
	log "PRIMARY NODE IS UP !"
}

function curlAuth {
	if [ -z "$ART_API_KEY" ]; then
		echo "curl -u$ART_LOGIN:$ART_PASSWORD"
	else
		echo "curl -H \"X-JFrog-Art-Api:$ART_API_KEY\""
	fi
}

function log {
	echo "[SECONDARY $NODE] $1"
}

function getLicenseFromPrimary {
	local arguments=" --show-error --silent --fail -X POST $ART_PRIMARY_BASE_URL/api/plugins/execute/getSecondaryLicense?params=$NODE"
	local curl=$(curlAuth)
	local response=$((eval $curl $arguments) 2>&1)
	local responseStatus=$?
	if [ $responseStatus -ne 0 ]; then
		log "Couldn't retrieve the license from the primary, got response from server $response "
		echo $responseStatus
	elif [ ! -z "$response" ]; then
		echo -n "$response" > /etc/opt/jfrog/artifactory/artifactory.lic
		echo "0"
	else
		log "License not found from primary"
		echo $responseStatus
	fi	
}

function getLicenseFromPrimaryOrDieTrying {
	local numberOfRetries=5
	local retry=0
	local sleepingTime=15
	while [ "$retry" -lt "$numberOfRetries" ]; do
		local lic=$(getLicenseFromPrimary)
		echo "return code from getLic : $lic"
		if [ "$lic" == "0" ]; then
			return 0
		fi
		retry=$((retry+1))
		log "Retry $retry getting license from primary in $sleepingTime seconds"
		sleep $sleepingTime
	done
	log "Have tried $numberOfRetries to get a license from primary without success, now exiting"
	exit 1
}

setPorts
setHaNodeProperties
setPrimaryBaseUrl
waitForPrimaryNode
getLicenseFromPrimaryOrDieTrying

/opt/jfrog/artifactory/bin/artifactory.sh

